/*
    Introduction to OOP with Java 3rd Ed, McGraw-Hill

    Wu/Otani

    Chapter 6 Sample Program: Test the random number generator

    File: Ch6TestRandomGenerator.java

*/

import javax.swing.*;
import java.util.*;

class Ch6TestRandomGenerator {

    private static final int CLUB    = 1;
    private static final int SPADE   = 2;
    private static final int HEART   = 3;
    private static final int DIAMOND = 4;

    public static void main( String[] args ) {

        Ch6TestRandomGenerator tester = new Ch6TestRandomGenerator();

        tester.start();
    }

    public void start( ) {

        long N;

        while (keepTesting()) {

            N = getSize();

            generate(N);
        }
    }

    private long getSize( ) {

        String inputStr;
        long    size;

        while (true) {
            inputStr = JOptionPane.showInputDialog(null, "Enter size:");

            size = Long.parseLong(inputStr);

            if (size > 0) break; //input okay so exit

            JOptionPane.showMessageDialog(null, "Input must be positive");
        }

        return size;
    }

    private void generate(long size) {

        Date startTime, endTime;
        int suit;
        long clubCnt, spadeCnt, heartCnt, diamondCnt;

        clubCnt = spadeCnt = heartCnt = diamondCnt = 0;

        startTime = new Date();

        for (int i = 0; i < size;  i++) {

            suit = getRandom(CLUB, DIAMOND);

            switch (suit) {
                case CLUB:    clubCnt++;
                              break;

                case SPADE:   spadeCnt++;
                              break;

                case HEART:   heartCnt++;
                              break;

                case DIAMOND: diamondCnt++;
                              break;

                default: //default case should never happen
                         JOptionPane.showMessageDialog(null,"Internal Error");
                         System.exit(0); //terminate the program
            }

        }

        endTime = new Date();

        System.out.println("N is " + size + "\n");
        System.out.println("Club:    " + clubCnt    + "    " + (double)clubCnt/size);
        System.out.println("Spade:   " + spadeCnt   + "    " + (double)spadeCnt/size);
        System.out.println("Heart:   " + heartCnt   + "    " + (double)heartCnt/size);
        System.out.println("Diamond: " + diamondCnt + "    " + (double)diamondCnt/size);

        double elapsedTimeInSec = (double) (endTime.getTime() - startTime.getTime())/1000.0;
        System.out.println("Elapsed time (sec): " + elapsedTimeInSec );

        System.out.println("\n");
    }


    private int getRandom(int min, int max) {

        int randomNumber = (int) (Math.floor(Math.random() * (max-min+1)) + min);

        return randomNumber;
    }


    private boolean keepTesting( ) {

        boolean result;

        int response = JOptionPane.showConfirmDialog(null,
                        /*prompt*/            "Perform Test?",
                        /*dialog title*/      "Random Number Generator",
                        /*button options*/    JOptionPane.YES_NO_OPTION);

        if (response == JOptionPane.YES_OPTION) {
            result = true;
        } else {
            result = false;
        }

        return result;
    }
}